"""qdrive_v_0.1.0

Revision ID: 4000a8b9703a
Revises: 
Create Date: 2023-12-07 10:14:14.721493

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '4000a8b9703a'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('schemas',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('uuid', sa.Uuid(), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('description', sa.String(), nullable=False),
    sa.Column('created', sa.DateTime(timezone=True), nullable=False),
    sa.Column('modified', sa.DateTime(timezone=True), nullable=False),
    sa.Column('schema', sa.JSON(), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('uuid')
    )
    op.create_table('users',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('username', sa.String(), nullable=False),
    sa.Column('firstname', sa.String(), nullable=False),
    sa.Column('lastname', sa.String(), nullable=False),
    sa.Column('email', sa.String(), nullable=False),
    sa.Column('created', sa.DateTime(timezone=True), nullable=False),
    sa.Column('modified', sa.DateTime(timezone=True), nullable=False),
    sa.Column('disable_on', sa.DateTime(), nullable=True),
    sa.Column('user_type', sa.Enum('admin', 'scope_admin', 'standard_user', 'superuser', name='usertype'), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('email'),
    sa.UniqueConstraint('username')
    )
    op.create_table('scopes',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('uuid', sa.Uuid(), nullable=False),
    sa.Column('created', sa.DateTime(timezone=True), nullable=False),
    sa.Column('modified', sa.DateTime(timezone=True), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('description', sa.String(), nullable=False),
    sa.Column('archived', sa.Boolean(), nullable=False),
    sa.Column('schema_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['schema_id'], ['schemas.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('uuid')
    )
    op.create_table('dataset_attr',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('key', sa.String(), nullable=False),
    sa.Column('value', sa.String(), nullable=False),
    sa.Column('scope_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['scope_id'], ['scopes.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('key', 'value', 'scope_id')
    )
    op.create_index(op.f('ix_dataset_attr_scope_id'), 'dataset_attr', ['scope_id'], unique=False)
    op.create_table('datasets',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('uuid', sa.Uuid(), nullable=False),
    sa.Column('alt_uid', sa.String(), nullable=True),
    sa.Column('collected', sa.TIMESTAMP(), nullable=False),
    sa.Column('created', sa.DateTime(timezone=True), nullable=False),
    sa.Column('modified', sa.DateTime(timezone=True), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('scope_id', sa.Integer(), nullable=False),
    sa.Column('creator', sa.String(), nullable=False),
    sa.Column('description', sa.String(), nullable=True),
    sa.Column('keywords', sa.JSON(), nullable=False),
    sa.Column('search_helper', sa.String(), nullable=False),
    sa.Column('ranking', sa.Integer(), nullable=False),
    sa.Column('synchronized', sa.Boolean(), nullable=False),
    sa.ForeignKeyConstraint(['scope_id'], ['scopes.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('uuid', 'alt_uid')
    )
    op.create_index(op.f('ix_datasets_alt_uid'), 'datasets', ['alt_uid'], unique=False)
    op.create_index(op.f('ix_datasets_scope_id'), 'datasets', ['scope_id'], unique=False)
    op.create_index(op.f('ix_datasets_uuid'), 'datasets', ['uuid'], unique=False)
    op.create_table('scope_user_link',
    sa.Column('scope', sa.Integer(), nullable=False),
    sa.Column('user_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['scope'], ['scopes.id'], ),
    sa.ForeignKeyConstraint(['user_id'], ['users.id'], ),
    sa.PrimaryKeyConstraint('scope', 'user_id')
    )
    op.create_table('ds_attr_link',
    sa.Column('dataset_id', sa.Integer(), nullable=False),
    sa.Column('dataset_attr_id', sa.Integer(), nullable=False),
    sa.ForeignKeyConstraint(['dataset_attr_id'], ['dataset_attr.id'], ),
    sa.ForeignKeyConstraint(['dataset_id'], ['datasets.id'], ),
    sa.PrimaryKeyConstraint('dataset_id', 'dataset_attr_id')
    )
    op.create_table('files',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('filename', sa.String(), nullable=False),
    sa.Column('file_generator', sa.String(), nullable=True),
    sa.Column('uuid', sa.Uuid(), nullable=False),
    sa.Column('version_id', sa.Integer(), nullable=False),
    sa.Column('creator', sa.String(), nullable=False),
    sa.Column('type', sa.Enum('HDF5', 'HDF5_NETCDF', 'NDARRAY', 'JSON', 'TEXT', 'UNKNOWN', name='filetype'), nullable=False),
    sa.Column('scope_id', sa.Integer(), nullable=False),
    sa.Column('dataset_id', sa.Integer(), nullable=False),
    sa.Column('collected', sa.DateTime(), nullable=False),
    sa.Column('created', sa.DateTime(timezone=True), nullable=False),
    sa.Column('modified', sa.DateTime(timezone=True), nullable=False),
    sa.Column('etag', sa.String(), nullable=True),
    sa.Column('size', sa.Integer(), nullable=False),
    sa.Column('status', sa.Enum('writing', 'complete', 'unavailable', name='filestatus'), nullable=False),
    sa.Column('local_path', sa.String(), nullable=True),
    sa.Column('S3_link', sa.String(), nullable=True),
    sa.Column('S3_validity', sa.DateTime(), nullable=True),
    sa.Column('last_accessed', sa.DateTime(), nullable=True),
    sa.ForeignKeyConstraint(['dataset_id'], ['datasets.id'], ),
    sa.ForeignKeyConstraint(['scope_id'], ['scopes.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('uuid', 'version_id')
    )
    op.create_table('core_tools_sync_data',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('database_id', sa.String(), nullable=False),
    sa.Column('core_tools_id', sa.Integer(), nullable=False),
    sa.Column('collected', sa.DateTime(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('database_id', 'core_tools_id', name='core_tools_id_constraint')
    )
    op.create_table('sync_sources',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(), nullable=False),
    sa.Column('type', sa.Enum('native', 'coretools', 'qcodes', 'quantify', name='syncsourcetypes'), nullable=False),
    sa.Column('status', sa.Enum('synchronized', 'synchronizing', 'pending', 'quantify', name='syncsourcestatus'), nullable=False),
    sa.Column('items_synchronized', sa.Integer(), nullable=False),
    sa.Column('items_total', sa.Integer(), nullable=False),
    sa.Column('last_update', sa.DateTime(), nullable=False),
    sa.Column('config_data', sa.JSON(), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('name')
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('sync_sources')
    op.drop_table('core_tools_sync_data')
    op.drop_table('files')
    op.drop_table('ds_attr_link')
    op.drop_table('scope_user_link')
    op.drop_index(op.f('ix_datasets_uuid'), table_name='datasets')
    op.drop_index(op.f('ix_datasets_scope_id'), table_name='datasets')
    op.drop_index(op.f('ix_datasets_alt_uid'), table_name='datasets')
    op.drop_table('datasets')
    op.drop_index(op.f('ix_dataset_attr_scope_id'), table_name='dataset_attr')
    op.drop_table('dataset_attr')
    op.drop_table('scopes')
    op.drop_table('users')
    op.drop_table('schemas')
    # ### end Alembic commands ###
