"""update database to support file sync

Revision ID: c68bfc310f78
Revises: f8337520d10d
Create Date: 2024-06-14 10:58:34.137786

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'c68bfc310f78'
down_revision: Union[str, None] = 'f8337520d10d'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    op.create_table('sync_items_temp',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('sync_source_id', sa.Integer(), nullable=False),
    sa.Column('dataIdentifier', sa.String(), nullable=False),
    sa.Column('scopeIdentifier', sa.String(), nullable=True),
    sa.Column('datasetUUID', sa.Uuid(), nullable=False),
    sa.Column('syncPriority', sa.Float(), nullable=False),
    sa.Column('synchronized', sa.Boolean(), nullable=False),
    sa.Column('attempts', sa.Integer(), nullable=False),
    sa.Column('last_update', sa.DateTime(), server_default=sa.text('(CURRENT_TIMESTAMP)'), nullable=False),
    sa.ForeignKeyConstraint(['sync_source_id'], ['sync_sources.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('datasetUUID'),
    sa.UniqueConstraint('sync_source_id', 'dataIdentifier', name='sync_source_did_constraint_sync_items')
    )
    op.execute("INSERT INTO sync_items_temp SELECT id, sync_source_id, dataIdentifier, scopeIdentifier, datasetUUID, dataIdentifier, synchronized, attempts, last_update FROM sync_items")
    
    op.drop_index('failed_upload_index', table_name='sync_items')
    op.drop_index('ix_sync_items_dataIdentifier', table_name='sync_items')
    op.drop_index('read_item_index', table_name='sync_items')
    op.drop_index('sync_upload_index', table_name='sync_items')
    
    op.drop_table('sync_items')
    
    op.execute("ALTER TABLE sync_items_temp RENAME TO sync_items;")
    
    op.create_index('failed_upload_index', 'sync_items', ['sync_source_id', 'attempts'], unique=False)
    op.create_index(op.f('ix_sync_items_dataIdentifier'), 'sync_items', ['dataIdentifier'], unique=False)
    op.create_index('read_item_index', 'sync_items', ['sync_source_id', 'scopeIdentifier', 'synchronized', 'dataIdentifier'], unique=False)
    op.create_index('sync_upload_index', 'sync_items', ['sync_source_id', 'attempts', 'synchronized'], unique=False)
    op.create_index('ix_sync_items_syncPriority', 'sync_items', ['syncPriority'])
    
def downgrade() -> None:
    raise NotImplementedError("Downgrade is not supported for this migration -- comment out this line if you want to try anyways...")
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('sync_items_temp',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('sync_source_id', sa.Integer(), nullable=False),
    sa.Column('dataIdentifier', sa.Integer(), nullable=False),
    sa.Column('scopeIdentifier', sa.String(), nullable=True),
    sa.Column('datasetUUID', sa.Uuid(), nullable=False),
    sa.Column('synchronized', sa.Boolean(), nullable=False),
    sa.Column('attempts', sa.Integer(), nullable=False),
    sa.Column('last_update', sa.DateTime(), server_default=sa.text('(CURRENT_TIMESTAMP)'), nullable=False),
    sa.ForeignKeyConstraint(['sync_source_id'], ['sync_sources.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('datasetUUID'),
    sa.UniqueConstraint('sync_source_id', 'dataIdentifier', name='sync_source_did_constraint_sync_items')
    )
    
    op.execute("INSERT INTO sync_items_temp SELECT id, sync_source_id, syncPriority, scopeIdentifier, datasetUUID, synchronized, attempts, last_update FROM sync_items")
    
    op.drop_index('failed_upload_index', table_name='sync_items')
    op.drop_index('ix_sync_items_dataIdentifier', table_name='sync_items')
    op.drop_index('read_item_index', table_name='sync_items')
    op.drop_index('sync_upload_index', table_name='sync_items')
    op.drop_index('ix_sync_items_syncPriority', table_name='sync_items')
    
    op.drop_table('sync_items')
    
    op.execute("ALTER TABLE sync_items_temp RENAME TO sync_items;")
    
    op.create_index('failed_upload_index', 'sync_items', ['sync_source_id', 'attempts'], unique=False)
    op.create_index(op.f('ix_sync_items_dataIdentifier'), 'sync_items', ['dataIdentifier'], unique=False)
    op.create_index('read_item_index', 'sync_items', ['sync_source_id', 'scopeIdentifier', 'synchronized', 'dataIdentifier'], unique=False)
    op.create_index('sync_upload_index', 'sync_items', ['sync_source_id', 'attempts', 'synchronized'], unique=False)