"""update_sync_tables

Revision ID: fe8c0d015ba2
Revises: c5d93c68feda
Create Date: 2024-03-12 09:05:57.140564

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'fe8c0d015ba2'
down_revision: Union[str, None] = 'c5d93c68feda'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    op.drop_index('failed_upload_index', table_name='core_tools_sync_data')
    op.drop_index('ix_core_tools_sync_data_core_tools_id', table_name='core_tools_sync_data')
    op.drop_index('read_item_index', table_name='core_tools_sync_data')
    op.drop_index('sync_upload_index', table_name='core_tools_sync_data')
    op.drop_table('core_tools_sync_data')

    op.create_table('sync_items',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('sync_source_id', sa.Integer(), nullable=False),
    sa.Column('dataIdentifier', sa.Integer(), nullable=False),
    sa.Column('scopeIdentifier', sa.String(), nullable=True),
    sa.Column('datasetUUID', sa.Uuid(), nullable=False),
    sa.Column('synchronized', sa.Boolean(), nullable=False),
    sa.Column('attempts', sa.Boolean(), nullable=False),
    sa.Column('last_update', sa.DateTime(), server_default=sa.text('(CURRENT_TIMESTAMP)'), nullable=False),
    sa.ForeignKeyConstraint(['sync_source_id'], ['sync_sources.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('datasetUUID'),
    sa.UniqueConstraint('sync_source_id', 'dataIdentifier', name='sync_source_did_constraint_sync_items')
    )
    op.create_index('failed_upload_index', 'sync_items', ['sync_source_id', 'attempts'], unique=False)
    op.create_index(op.f('ix_sync_items_dataIdentifier'), 'sync_items', ['dataIdentifier'], unique=False)
    op.create_index('read_item_index', 'sync_items', ['sync_source_id', 'scopeIdentifier', 'synchronized', 'dataIdentifier'], unique=False)
    op.create_index('sync_upload_index', 'sync_items', ['sync_source_id', 'attempts', 'synchronized'], unique=False)
    op.create_table('sync_scope_mappings',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('sync_source_id', sa.Integer(), nullable=False),
    sa.Column('scope_identifier', sa.String(), nullable=False),
    sa.Column('scope_uuid', sa.Uuid(), nullable=False),
    sa.ForeignKeyConstraint(['sync_source_id'], ['sync_sources.id'], ),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('sync_source_id', 'scope_identifier', name='sync_source_did_constraint_scope_mapping')
    )
    op.add_column('sync_sources', sa.Column('default_scope', sa.Uuid(), nullable=True))
    op.add_column('sync_sources', sa.Column('auto_mapping', sa.Boolean(), nullable=False, server_default='0'))
    op.execute('UPDATE sync_sources SET auto_mapping = true WHERE type = "coretools"')

def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('sync_scope_mappings')
    op.drop_index('sync_upload_index', table_name='sync_items')
    op.drop_index('read_item_index', table_name='sync_items')
    op.drop_index(op.f('ix_sync_items_dataIdentifier'), table_name='sync_items')
    op.drop_index('failed_upload_index', table_name='sync_items')
    op.drop_table('sync_items')

    op.drop_column('sync_sources', 'default_scope')
    op.drop_column('sync_sources', 'auto_mapping')
    op.execute('DELETE FROM sync_sources WHERE type = "qcodes"')
    op.execute('DELETE FROM sync_sources WHERE type = "quantify"')
    op.create_table('core_tools_sync_data',
    sa.Column('id', sa.INTEGER(), nullable=False),
    sa.Column('database_id', sa.VARCHAR(), nullable=False),
    sa.Column('core_tools_id', sa.INTEGER(), nullable=False),
    sa.Column('collected', sa.DATETIME(), nullable=True),
    sa.Column('core_tools_project', sa.VARCHAR(), nullable=False),
    sa.Column('synchronized', sa.BOOLEAN(), nullable=False),
    sa.Column('failed', sa.BOOLEAN(), nullable=False),
    sa.Column('last_update', sa.DATETIME(), server_default=sa.text("'2024-03-12 08:58:44.696603'"), nullable=False),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('database_id', 'core_tools_id', name='core_tools_id_constraint')
    )
    op.create_index('sync_upload_index', 'core_tools_sync_data', ['database_id', 'failed', 'synchronized'], unique=False)
    op.create_index('read_item_index', 'core_tools_sync_data', ['database_id', 'core_tools_project', 'synchronized', 'core_tools_id'], unique=False)
    op.create_index('ix_core_tools_sync_data_core_tools_id', 'core_tools_sync_data', ['core_tools_id'], unique=False)
    op.create_index('failed_upload_index', 'core_tools_sync_data', ['database_id', 'failed'], unique=False)
    # ### end Alembic commands ###
